@@ -120,8 +120,8 @@ from vcstools.vcs_abstraction import get_vcs_client
 
 try:
     import rosdistro
-    if parse_version(rosdistro.__version__) < parse_version('0.4.0'):
-        error("rosdistro version 0.4.0 or greater is required, found '{0}' from '{1}'."
+    if parse_version(rosdistro.__version__) < parse_version('0.7.0'):
+        error("rosdistro version 0.7.0 or greater is required, found '{0}' from '{1}'."
               .format(rosdistro.__version__, os.path.dirname(rosdistro.__file__)),
               exit=True)
 except ImportError:
@@ -197,9 +197,10 @@ def get_index_url():
             info("ROS Distro index file associate with commit '{0}'"
                  .format(_rosdistro_index_commit))
             # Also mutate the index_url to use the commit (rather than the moving branch name)
-            base_org, base_repo, base_branch, base_path = get_gh_info(index_url)
+            base_info = get_gh_info(index_url)
+            base_branch = base_info['branch']
             rosdistro_index_commit = _rosdistro_index_commit  # Copy global into local for substitution
-            middle = "{base_org}/{base_repo}".format(**locals())
+            middle = "{org}/{repo}".format(**base_info)
             index_url = index_url.replace("{pr.netloc}/{middle}/{base_branch}/".format(**locals()),
                                           "{pr.netloc}/{middle}/{rosdistro_index_commit}/".format(**locals()))
             info("New ROS Distro index url: '{0}'".format(index_url))
@@ -217,7 +218,7 @@ def get_index():
             error("This version of bloom does not support rosdistro version "
                   "'{0}', please use an older version of bloom."
                   .format(_rosdistro_index.version), exit=True)
-        if _rosdistro_index.version > 3:
+        if _rosdistro_index.version > 4:
             error("This version of bloom does not support rosdistro version "
                   "'{0}', please update bloom.".format(_rosdistro_index.version), exit=True)
     return _rosdistro_index
@@ -227,7 +228,12 @@ def list_distributions():
     return sorted(get_index().distributions.keys())
 
 
-def get_most_recent(thing_name, repository):
+def get_distribution_type(distro):
+    return get_index().distributions[distro].get('distribution_type')
+
+
+def get_most_recent(thing_name, repository, reference_distro):
+    reference_distro_type = get_distribution_type(reference_distro)
     distros_with_entry = {}
     get_things = {
         'release': lambda r: None if r.release_repository is None else r.release_repository,
@@ -236,6 +242,10 @@ def get_most_recent(thing_name, repository):
     }
     get_thing = get_things[thing_name]
     for distro in list_distributions():
+        # skip distros with a different type if the information is available
+        if reference_distro_type is not None:
+            if get_distribution_type(distro) != reference_distro_type:
+                continue
         distro_file = get_distribution_file(distro)
         if repository in distro_file.repositories:
             thing = get_thing(distro_file.repositories[repository])
@@ -344,7 +354,7 @@ def get_repo_uri(repository, distro):
         info("Here is the url for a typical release repository on GitHub: https://github.com/ros-gbp/rviz-release.git")
         # Calculate a reasonable default from the list of previous distros
         info(fmt("@{gf}@!==> @|") + "Looking for a release of this repository in a different distribution...")
-        default_distro, default_release = get_most_recent('release', repository)
+        default_distro, default_release = get_most_recent('release', repository, distro)
         default_release_repo_url = default_release.url if default_release else "press enter to abort"
         if default_distro is not None:
             warning("A different distribution, '{0}', released this repository.".format(default_distro))
@@ -422,10 +432,24 @@ def get_relative_distribution_file_path(distro):
 
 
 def generate_release_tag(distro):
-    return ('release/%s/{package}/{version}' % distro).encode('utf-8')
+    tag = ('release/%s/{package}/{version}' % distro)
+    if sys.version_info[0] < 3:
+        tag == tag.encode('utf-8')
+    return tag
 
 
 def generate_ros_distro_diff(track, repository, distro, override_release_repository_url):
+    def convert_unicode_dict_to_str(d):
+        for key, value in d.items():
+            if type(key) == unicode:
+                del d[key]
+                key = key.encode('utf-8')
+            if type(value) == unicode:
+                value = value.encode('utf-8')
+            if type(value) == dict:
+                convert_unicode_dict_to_str(value)
+            d[key] = value
+
     global _user_provided_release_url
     distribution_dict = get_distribution_file(distro).get_data()
     # Get packages
@@ -436,26 +460,28 @@ def generate_ros_distro_diff(track, repository, distro, override_release_reposit
     track_dict = get_tracks_dict_raw()['tracks'][track]
     last_version = track_dict['last_version']
     release_inc = track_dict['release_inc']
-    version = '{0}-{1}'.format(last_version, release_inc).encode('utf-8')
+    version = '{0}-{1}'.format(last_version, release_inc)
     # Create a repository if there isn't already one
     if repository not in distribution_dict['repositories']:
         distribution_dict['repositories'][repository] = {}
     # Create a release entry if there isn't already one
     if 'release' not in distribution_dict['repositories'][repository]:
-        distribution_dict['repositories'][repository]['release'.encode('utf-8')] = {
-            'url'.encode('utf-8'): override_release_repository_url or _user_provided_release_url
+        distribution_dict['repositories'][repository]['release'] = {
+            'url': override_release_repository_url or _user_provided_release_url
         }
     # Update the repository
     repo = distribution_dict['repositories'][repository]['release']
     # Consider the override
     if override_release_repository_url is not None:
-        repo['url'.encode('utf-8')] = override_release_repository_url
+        repo['url'] = override_release_repository_url
     if 'tags' not in repo:
-        repo['tags'.encode('utf-8')] = {}
-    repo['tags']['release'.encode('utf-8')] = generate_release_tag(distro)
-    repo['version'.encode('utf-8')] = version
+        repo['tags'] = {}
+    repo['tags']['release'] = generate_release_tag(distro)
+    repo['version'] = version
+    if 'last_release' in track_dict:
+        repo['upstream_tag'] = track_dict['last_release']
     if 'packages' not in repo:
-        repo['packages'.encode('utf-8')] = []
+        repo['packages'] = []
     for path, pkg in packages.items():
         if pkg.name not in repo['packages']:
             repo['packages'].append(pkg.name)
@@ -465,6 +491,8 @@ def generate_ros_distro_diff(track, repository, distro, override_release_reposit
         if pkg_name not in packages_being_released:
             repo['packages'].remove(pkg_name)
     repo['packages'].sort()
+    if sys.version_info[0] < 3:
+        convert_unicode_dict_to_str(repo)
 
     def get_repository_info_from_user(url_type, defaults=None):
         data = {}
@@ -518,7 +546,7 @@ def generate_ros_distro_diff(track, repository, distro, override_release_reposit
         if not docs and maybe_continue(msg='Would you like to add documentation information for this repository?'):
             defaults = None
             info(fmt("@{gf}@!==> @|") + "Looking for a doc entry for this repository in a different distribution...")
-            default_distro, default_doc = get_most_recent('doc', repository)
+            default_distro, default_doc = get_most_recent('doc', repository, distro)
             if default_distro is None:
                 warning("No existing doc entries found for use as defaults.")
             else:
@@ -541,7 +569,7 @@ def generate_ros_distro_diff(track, repository, distro, override_release_reposit
             defaults = None
             info(fmt("@{gf}@!==> @|") +
                  "Looking for a source entry for this repository in a different distribution...")
-            default_distro, default_source = get_most_recent('source', repository)
+            default_distro, default_source = get_most_recent('source', repository, distro)
             if default_distro is None:
                 warning("No existing source entries found for use as defaults.")
             else:
@@ -601,7 +629,7 @@ def generate_ros_distro_diff(track, repository, distro, override_release_reposit
     distro_file_name = get_relative_distribution_file_path(distro)
     updated_distribution_file = rosdistro.DistributionFile(distro, distribution_dict)
     distro_dump = yaml_from_distribution_file(updated_distribution_file)
-    distro_file_raw = load_url_to_file_handle(get_distribution_file_url(distro)).read()
+    distro_file_raw = load_url_to_file_handle(get_distribution_file_url(distro)).read().decode('utf-8')
     if distro_file_raw != distro_dump:
         # Calculate the diff
         udiff = difflib.unified_diff(distro_file_raw.splitlines(), distro_dump.splitlines(),
@@ -629,7 +657,7 @@ def generate_ros_distro_diff(track, repository, distro, override_release_reposit
                 udiff_raw += line
             info(line, use_prefix=False, end='')
         # Assert that only this repository is being changed
-        distro_file_yaml = yaml.load(distro_file_raw)
+        distro_file_yaml = yaml.safe_load(distro_file_raw)
         distro_yaml = yaml.load(distro_dump)
         if 'repositories' in distro_file_yaml:
             distro_file_repos = distro_file_yaml['repositories']
@@ -654,11 +682,21 @@ def generate_ros_distro_diff(track, repository, distro, override_release_reposit
 def get_gh_info(url):
     o = urlparse(url)
     if 'raw.github.com' not in o.netloc and 'raw.githubusercontent.com' not in o.netloc:
-        return None, None, None, None
+        return None
     url_paths = o.path.split('/')
     if len(url_paths) < 5:
-        return None, None, None, None
-    return url_paths[1], url_paths[2], url_paths[3], '/'.join(url_paths[4:])
+        return None
+    return {'server': 'github.com',
+            'org': url_paths[1],
+            'repo': url_paths[2],
+            'branch': url_paths[3],
+            'path': '/'.join(url_paths[4:])}
+
+
+def get_repo_info(distro_url):
+    gh_info = get_gh_info(distro_url)
+    if gh_info:
+        return gh_info
 
 
 _gh = None
@@ -670,7 +708,7 @@ def get_github_interface(quiet=False):
         warning("Receiving 401 when trying to create an oauth token can be caused by the user "
                 "having two-factor authentication enabled.")
         warning("If 2FA is enabled, the user will have to create an oauth token manually.")
-        warning("A token can be created at https://github.com/settings/applications")
+        warning("A token can be created at https://github.com/settings/tokens")
         warning("The resulting token can be placed in the '{oauth_config_path}' file as such:"
                 .format(**locals()))
         info("")
@@ -721,7 +759,7 @@ def get_github_interface(quiet=False):
         gh = Github(username, auth=auth_header_from_basic_auth(username, password))
         try:
             token = gh.create_new_bloom_authorization(update_auth=True)
-            with open(oauth_config_path, 'a') as f:
+            with open(oauth_config_path, 'w') as f:
                 config.update({'oauth_token': token, 'github_user': username})
                 f.write(json.dumps(config))
             info("The token '{token}' was created and stored in the bloom config file: '{oauth_config_path}'"
@@ -790,61 +828,20 @@ def open_pull_request(track, repository, distro, interactive, override_release_r
         return None
     version = updated_distribution_file.repositories[repository].release_repository.version
     updated_distro_file_yaml = yaml_from_distribution_file(updated_distribution_file)
-    # Determine if the distro file is hosted on github...
-    base_org, base_repo, base_branch, base_path = get_gh_info(get_distribution_file_url(distro))
-    if None in [base_org, base_repo, base_branch, base_path]:
+
+    # Determine where the distro file is hosted...
+    distro_url = get_distribution_file_url(distro)
+    base_info = get_repo_info(distro_url)
+    if not base_info:
         warning("Automated pull request only available via github.com")
         return
+
     # If we did replace the branch in the url with a commit, restore that now
     if _rosdistro_index_original_branch is not None:
-        base_branch = _rosdistro_index_original_branch
-    # Get the github interface
-    gh = get_github_interface()
-    if gh is None:
-        return None
-    # Determine the head org/repo for the pull request
-    head_org = gh.username  # The head org will always be gh user
-    head_repo = None
-    # Check if the github user and the base org are the same
-    if gh.username == base_org:
-        # If it is, then a fork is not necessary
-        head_repo = gh.get_repo(base_org, base_repo)
-    else:
-        info(fmt("@{bf}@!==> @|@!Checking on GitHub for a fork to make the pull request from..."))
-        # It is not, so a fork will be required
-        # Check if a fork already exists on the user's account
+        base_info['branch'] = _rosdistro_index_original_branch
 
-        try:
-            repo_forks = gh.list_forks(base_org, base_repo)
-            user_forks = [r for r in repo_forks if r.get('owner', {}).get('login', '') == gh.username]
-            # github allows only 1 fork per org as far as I know. We just take the first one.
-            head_repo = user_forks[0] if user_forks else None
-
-        except GithubException as exc:
-            debug("Received GithubException while checking for fork: {exc}".format(**locals()))
-            pass  # 404 or unauthorized, but unauthorized should have been caught above
-
-        # If not head_repo still, a fork does not exist and must be created
-        if head_repo is None:
-            warning("Could not find a fork of {base_org}/{base_repo} on the {gh.username} GitHub account."
-                    .format(**locals()))
-            warning("Would you like to create one now?")
-            if not maybe_continue():
-                warning("Skipping the pull request...")
-                return
-            # Create a fork
-            try:
-                head_repo = gh.create_fork(base_org, base_repo)  # Will raise if not successful
-            except GithubException as exc:
-                error("Aborting pull request: {0}".format(exc))
-                return
-    head_repo = head_repo.get('name', '')
-    info(fmt("@{bf}@!==> @|@!" +
-             "Using this fork to make a pull request from: {head_org}/{head_repo}".format(**locals())))
-    # Clone the fork
-    info(fmt("@{bf}@!==> @|@!" + "Cloning {0}/{1}...".format(head_org, head_repo)))
-    new_branch = None
-    title = "{0}: {1} in '{2}' [bloom]".format(repository, version, base_path)
+    # Create content for PR
+    title = "{0}: {1} in '{2}' [bloom]".format(repository, version, base_info['path'])
     track_dict = get_tracks_dict_raw()['tracks'][track]
     body = u"""\
 Increasing version of package(s) in repository `{repository}` to `{version}`:
@@ -858,56 +855,109 @@ Increasing version of package(s) in repository `{repository}` to `{version}`:
         repository=repository,
         orig_version=orig_version or 'null',
         version=version,
-        distro_file=base_path,
+        distro_file=base_info['path'],
         bloom_version=bloom.__version__,
         upstream_repo=track_dict['vcs_uri'],
         release_repo=updated_distribution_file.repositories[repository].release_repository.url,
     )
     body += get_changelog_summary(generate_release_tag(distro))
-    with temporary_directory() as temp_dir:
-        def _my_run(cmd, msg=None):
-            if msg:
-                info(fmt("@{bf}@!==> @|@!" + sanitize(msg)))
-            else:
-                info(fmt("@{bf}@!==> @|@!" + sanitize(str(cmd))))
-            from subprocess import check_call
-            check_call(cmd, shell=True)
-        # Use the oauth token to clone
-        rosdistro_url = 'https://{gh.token}:x-oauth-basic@github.com/{base_org}/{base_repo}.git'.format(**locals())
-        rosdistro_fork_url = 'https://{gh.token}:x-oauth-basic@github.com/{head_org}/{head_repo}.git'.format(**locals())
-        _my_run('mkdir -p {base_repo}'.format(**locals()))
-        with change_directory(base_repo):
-            _my_run('git init')
-            branches = [x['name'] for x in gh.list_branches(head_org, head_repo)]
-            new_branch = 'bloom-{repository}-{count}'
-            count = 0
-            while new_branch.format(repository=repository, count=count) in branches:
-                count += 1
-            new_branch = new_branch.format(repository=repository, count=count)
-            # Final check
-            info(fmt("@{cf}Pull Request Title: @{yf}" + sanitize(title)))
-            info(fmt("@{cf}Pull Request Body : \n@{yf}" + sanitize(body)))
-            msg = fmt("@!Open a @|@{cf}pull request@| @!@{kf}from@| @!'@|@!@{bf}" +
-                      "{head_org}/{head_repo}:{new_branch}".format(**locals()) +
-                      "@|@!' @!@{kf}into@| @!'@|@!@{bf}" +
-                      "{base_org}/{base_repo}:{base_branch}".format(**locals()) +
-                      "@|@!'?")
-            info(msg)
-            if interactive and not maybe_continue():
-                warning("Skipping the pull request...")
-                return
-            _my_run('git checkout -b {new_branch}'.format(**locals()))
-            _my_run('git pull {rosdistro_url} {base_branch}'.format(**locals()), "Pulling latest rosdistro branch")
-            if _rosdistro_index_commit is not None:
-                _my_run('git reset --hard {_rosdistro_index_commit}'.format(**globals()))
-            with open('{0}'.format(base_path), 'w') as f:
-                info(fmt("@{bf}@!==> @|@!Writing new distribution file: ") + str(base_path))
-                f.write(updated_distro_file_yaml)
-            _my_run('git add {0}'.format(base_path))
-            _my_run('git commit -m "{0}"'.format(title))
-            _my_run('git push {rosdistro_fork_url} {new_branch}'.format(**locals()), "Pushing changes to fork")
-    # Open the pull request
-    return gh.create_pull_request(base_org, base_repo, base_branch, head_org, new_branch, title, body)
+
+    if base_info['server'] == 'github.com':
+        # Get the github interface
+        gh = get_github_interface()
+        if gh is None:
+            return None
+        # Determine the head org/repo for the pull request
+        head_org = gh.username  # The head org will always be gh user
+        head_repo = None
+        base_repo_id = '{org}/{repo}'.format(**base_info)
+        # Check if the github user and the base org are the same
+        if gh.username == base_info['org']:
+            # If it is, then a fork is not necessary
+            head_repo = gh.get_repo(base_info['org'], base_info['repo'])
+        else:
+            info(fmt("@{bf}@!==> @|@!Checking on GitHub for a fork to make the pull request from..."))
+            # It is not, so a fork will be required
+            # Check if a fork already exists on the user's account
+
+            try:
+                repo_forks = gh.list_forks(base_info['org'], base_info['repo'])
+                user_forks = [r for r in repo_forks if r.get('owner', {}).get('login', '') == gh.username]
+                # github allows only 1 fork per org as far as I know. We just take the first one.
+                head_repo = user_forks[0] if user_forks else None
+
+            except GithubException as exc:
+                debug("Received GithubException while checking for fork: {exc}".format(**locals()))
+                pass  # 404 or unauthorized, but unauthorized should have been caught above
+
+            # If not head_repo still, a fork does not exist and must be created
+            if head_repo is None:
+                warning("Could not find a fork of {base_repo_id} on the {gh.username} GitHub account."
+                        .format(**locals()))
+                warning("Would you like to create one now?")
+                if not maybe_continue():
+                    warning("Skipping the pull request...")
+                    return
+                # Create a fork
+                try:
+                    head_repo = gh.create_fork(base_info['org'], base_info['repo'])  # Will raise if not successful
+                except GithubException as exc:
+                    error("Aborting pull request: {0}".format(exc))
+                    return
+        head_repo = head_repo.get('name', '')
+        info(fmt("@{bf}@!==> @|@!" +
+                 "Using this fork to make a pull request from: {head_org}/{head_repo}".format(**locals())))
+        # Clone the fork
+        info(fmt("@{bf}@!==> @|@!" + "Cloning {0}/{1}...".format(head_org, head_repo)))
+        new_branch = None
+
+        with temporary_directory() as temp_dir:
+            def _my_run(cmd, msg=None):
+                if msg:
+                    info(fmt("@{bf}@!==> @|@!" + sanitize(msg)))
+                else:
+                    info(fmt("@{bf}@!==> @|@!" + sanitize(str(cmd))))
+                from subprocess import check_call
+                check_call(cmd, shell=True)
+            # Use the oauth token to clone
+            rosdistro_url = 'https://{gh.token}:x-oauth-basic@github.com/{base_repo_id}.git'.format(**locals())
+            fork_template = 'https://{gh.token}:x-oauth-basic@github.com/{head_org}/{head_repo}.git'
+            rosdistro_fork_url = fork_template.format(**locals())
+            _my_run("mkdir -p {base_info[repo]}".format(**locals()))
+            with change_directory(base_info['repo']):
+                _my_run('git init')
+                branches = [x['name'] for x in gh.list_branches(head_org, head_repo)]
+                new_branch = 'bloom-{repository}-{count}'
+                count = 0
+                while new_branch.format(repository=repository, count=count) in branches:
+                    count += 1
+                new_branch = new_branch.format(repository=repository, count=count)
+                # Final check
+                info(fmt("@{cf}Pull Request Title: @{yf}" + sanitize(title)))
+                info(fmt("@{cf}Pull Request Body : \n@{yf}" + sanitize(body)))
+                msg = fmt("@!Open a @|@{cf}pull request@| @!@{kf}from@| @!'@|@!@{bf}" +
+                          "{head_org}/{head_repo}:{new_branch}".format(**locals()) +
+                          "@|@!' @!@{kf}into@| @!'@|@!@{bf}" +
+                          "{base_repo_id}:{base_info[branch]}".format(**locals()) +
+                          "@|@!'?")
+                info(msg)
+                if interactive and not maybe_continue():
+                    warning("Skipping the pull request...")
+                    return
+                _my_run('git checkout -b {new_branch}'.format(**locals()))
+                _my_run("git pull {rosdistro_url} {base_info[branch]}".format(**locals()),
+                        "Pulling latest rosdistro branch")
+                if _rosdistro_index_commit is not None:
+                    _my_run('git reset --hard {_rosdistro_index_commit}'.format(**globals()))
+                with open('{0}'.format(base_info['path']), 'w') as f:
+                    info(fmt("@{bf}@!==> @|@!Writing new distribution file: ") + str(base_info['path']))
+                    f.write(updated_distro_file_yaml)
+                _my_run('git add {0}'.format(base_info['path']))
+                _my_run('git commit -m "{0}"'.format(title))
+                _my_run('git push {rosdistro_fork_url} {new_branch}'.format(**locals()), "Pushing changes to fork")
+        # Open the pull request
+        return gh.create_pull_request(base_info['org'], base_info['repo'], base_info['branch'],
+                                      head_org, new_branch, title, body)
 
 _original_version = None
 
